-- [General variables]
local playerAircraftName = 'Tactical Aviation Aircraft #1' -- Input player aircraft name here
local enemyAircraftNames = {
  "Enemy AWACS", 
  "Enemy AWACS Escort #1", 
  "Enemy AWACS Escort #2", 
  "Enemy CAP Aircraft #1", 
  "Enemy CAP Aircraft #2", 
  "Enemy Strike Escort #1", 
  "Enemy Strike Escort #2", 
  "Enemy Strike Aircraft #1", 
  "Enemy Strike Aircraft #2", 
  "Enemy Strike Aircraft 2 #1", 
  "Enemy Strike Aircraft 2 #2", 
  "Enemy CAP Aircraft 2 #1", 
  "Enemy CAP Aircraft 2 #2",
  "Enemy Reinforcement Fighters #1",
  "Enemy Reinforcement Fighters #2"
  } -- Input enemy aircraft names from the mission here

-- [AGL function]
local minimumStealthAltitude = 150 -- This will vary how high the player can go before losing stealth
local altitudeViolationMax = 5 -- How long the player can go above the minimum altitude before losing stealth

-- [Visibility zones, each value describes how far out the respective zone goes (in meters) / The angles each zone covers]
local greenZoneDist = 7408 -- 4 NMi
local orangeZoneDist = 2778 -- 1.5 NMi
local redZoneDist = 926 -- 0.5 NMi
local greenZoneMaxAngle = 45
local orangeZoneMaxAngle = 135 -- The red zone is covered by simply assigning its angles to be greater than the maximum orange zone angle

-- [Cooldown period, how long it takes after shooting a radar missile/shooting an aircraft before the stealth mechanic can kick in again]
local cooldownPeriod = 60


-- AGL ALTITUDE VIOLATION CHECKER
local aboveAltitude = false
local altitudeViolationTimer = 0
local countdownBegin = 0
local withinStealthAltitude = false
function altitudeViolationChecker()
  local playerAircraft = Unit.getByName(playerAircraftName)
  if playerAircraft then
   local playerPosition = playerAircraft:getPoint()
   local terrainAltitude = land.getHeight({ x = playerPosition.x, y = playerPosition.z })
   local altAGL = playerPosition.y - terrainAltitude
     if altAGL > minimumStealthAltitude then
      if not aboveAltitude then
       aboveAltitude = true
       countdownBegin = timer.getTime()
      else
       local timeCount = timer.getTime() - countdownBegin
         if timeCount > altitudeViolationMax and altAGL > minimumStealthAltitude then
          withinStealthAltitude = false
         end
      end
       else
        aboveAltitude = false
        withinStealthAltitude = true
       end
    end
    timer.scheduleFunction(altitudeViolationChecker, {}, timer.getTime() + 1)
end
altitudeViolationChecker()



-- VISIBILITY ZONES
local resultsTable = {}
local keyWithSmallestDistance
local outsideVisibilityZones = false
function distanceBetweenAircrafts()
  local playerAircraft = Unit.getByName(playerAircraftName)
  if playerAircraft then
   local playerPosition = playerAircraft:getPosition()
   resultsTable = {} 
   for _, enemyName in ipairs(enemyAircraftNames) do
     local enemyAircraft = Unit.getByName(enemyName)
       if enemyAircraft and playerAircraft then
         local enemyPosition = enemyAircraft:getPosition()
         local x1, z1 = playerPosition.p.x, playerPosition.p.z
         local x2, z2 = enemyPosition.p.x, enemyPosition.p.z
         local xdist = x1 - x2
         local zdist = z1 - z2
         local distance = math.sqrt(xdist^2 + zdist^2)
         table.insert(resultsTable, {name = enemyName, distance = distance})
        end
    end
    timer.scheduleFunction(distanceBetweenAircrafts, {}, timer.getTime() + 1)
    return distance
  end
end
distanceBetweenAircrafts()

function findKeyWithSmallestDistance()
   local smallestDistance = math.huge
   keyWithSmallestDistance = nil
   for _, result in ipairs(resultsTable) do
    if result.distance then
     if result.distance < smallestDistance then
      smallestDistance = result.distance
      keyWithSmallestDistance = result.name
     end
    end
   end
end
findKeyWithSmallestDistance()

function visibilityZones()
  if keyWithSmallestDistance then
  local playerAircraft = Unit.getByName(playerAircraftName)
   local enemyAircraft = Unit.getByName(keyWithSmallestDistance) -- ID which aircraft we're working with
  if playerAircraft and enemyAircraft then
   local playerPosition = playerAircraft:getPosition()
   local enemyPosition = enemyAircraft:getPosition() -- Location of both aircraft
   local x1 = playerPosition.p.x
   local x2 = enemyPosition.p.x
   local z1 = playerPosition.p.z
   local z2 = enemyPosition.p.z -- X/Z Location of both aircraft
   
   local xdist = math.abs(x1 - x2)
   local zdist = math.abs(z1 - z2)
   
   local enemyHeadingRad = math.atan2(enemyPosition.x.z, enemyPosition.x.x) 
   local enemyHeadingDeg = math.deg(enemyHeadingRad) -- Enemy heading
   local angleRad = math.atan2(z1 - z2, x1 - x2) -- The actual math to find the angle between them
   local angleDeg = math.deg(angleRad) -- To degrees
   
   local distanceToEnemy = math.sqrt(xdist^2 + zdist^2)

   local noseAngle = math.abs(enemyHeadingDeg - angleDeg) 
    if noseAngle > 180 then
      noseAngle = noseAngle - 360
    else
      noseAngle = noseAngle 
    end
    if noseAngle < 0 then
      noseAngle = math.abs(noseAngle)
    end -- Makes sure the nose angle is working correctly and not becoming negative
    
   if distanceToEnemy < greenZoneDist and noseAngle < greenZoneMaxAngle then
     outsideVisibilityZones = false
   end
   if distanceToEnemy > greenZoneDist and noseAngle < greenZoneMaxAngle then
     outsideVisibilityZones = true
   end
   
   if distanceToEnemy < orangeZoneDist and noseAngle < orangeZoneMaxAngle and noseAngle > greenZoneMaxAngle then
     outsideVisibilityZones = false
   end
   if distanceToEnemy > orangeZoneDist and noseAngle < orangeZoneMaxAngle and noseAngle > greenZoneMaxAngle then
     outsideVisibilityZones = true
   end
   
   if distanceToEnemy < redZoneDist and noseAngle < 180 and noseAngle > orangeZoneMaxAngle then
     outsideVisibilityZones = false
   end
   
   if distanceToEnemy > redZoneDist and noseAngle < 180 and noseAngle > orangeZoneMaxAngle then
     outsideVisibilityZones = true
   end
   
   timer.scheduleFunction(visibilityZones, {}, timer.getTime() + 1) -- So it repeats
   return noseAngle
  end
  end
end 
visibilityZones()



-- COOLDOWNS
local stealth = false
local e = {}
local countdownbegin = 0
local playerAttackedSomeone = false
local timeSinceAttack = 0
function e:onEvent(event)
  if event.id == 1 then
   local attacker = event.initiator:getName()
   local MissileFired = event.weapon:getTypeName()
    if attacker == playerAircraftName and (MissileFired == "P_27P" or MissileFired == "P_27PE") then
     playerAttackedSomeone = true
     countdownbegin = timer.getTime()
    end
  end
    
  if event.id == 2 then
   local attacker = event.initiator:getName()
    if attacker and attacker == playerAircraftName then
     playerAttackedSomeone = true
     countdownbegin = timer.getTime()
    end
  end
end

function cooldownInitiate()
  if outsideVisibilityZones == true and withinStealthAltitude == true and playerAttackedSomeone == false then
   stealth = true
  end
  if outsideVisibilityZones == true and withinStealthAltitude == true and playerAttackedSomeone == true then
   stealth = false
   timeSinceAttack = timer.getTime() - countdownbegin
  end
  if outsideVisibilityZones == true and withinStealthAltitude == false and playerAttackedSomeone == true then
   stealth = false
  end
  if outsideVisibilityZones == false and withinStealthAltitude == true and playerAttackedSomeone == true then
   stealth = false
  end
  
  if outsideVisibilityZones == false and withinStealthAltitude == false and playerAttackedSomeone == true then
   stealth = false
  end
  if outsideVisibilityZones == false and withinStealthAltitude == true and playerAttackedSomeone == false then
   stealth = false
  end
  if outsideVisibilityZones == true and withinStealthAltitude == false and playerAttackedSomeone == false then
   stealth = false
  end
 
 
  if playerAttackedSomeone == true and timeSinceAttack > cooldownPeriod then
   playerAttackedSomeone = false
   timeSinceAttack = 0
  end

  timer.scheduleFunction(cooldownInitiate, {}, timer.getTime() + 1)
   if stealth == true then
     -- trigger.action.outText("Stealth ON! | " .. tostring(outsideVisibilityZones) .. " | " .. tostring(keyWithSmallestDistance), 1, true)
     trigger.action.setUserFlag("StealthMechanic", true)
   else 
     -- trigger.action.outText("Stealth OFF! | " .. tostring(outsideVisibilityZones) .. " | " .. tostring(keyWithSmallestDistance), 1, true)
     trigger.action.setUserFlag("StealthMechanic", false)
   end
end
cooldownInitiate()

world.addEventHandler(e)